/*------------------------------------------------------------------------------*/
/* IrSend.ino :  Copyright(C) BugMaker                                          */
/*      Version 0.00  2011.07.07                                                */
/*------------------------------------------------------------------------------*/
/*
◆機能
 赤外線リモコンの信号を出力するライブラリ。

◆Hardware
 Arduino2009 + LED

◆要素技術
 Ir送信

◆注意事項

◆Program memo
  LEDを接続する。Highレベル出力のときに点灯すること。

*/

/*------------- INCLUDE FILES ------------------------------------------*/
#include "Arduino.h"
#include "IrSend.h"


/*------------- FUNCTIONS ----------------------------------------------*/
IrSend::IrSend( byte port )
{
	led = port ;
	pinMode( led, OUTPUT ) ;
}

void IrSend::write( byte *p )
{
	switch( *p ) {
	case KASEIKYO:
		sendKase( p+1 ) ;
		break ;

	case NEC:
		sendNec( p+1 ) ;
		break ;

	case NEC_REPEAT:
		sendNecRepeat() ;
		break ;

	case SONY12:
		sendSony( p+1, 12 ) ;
		break ;

	case SONY15:
		sendSony( p+1, 15 ) ;
		break ;

	case SONY20:
		sendSony( p+1, 20 ) ;
		break ;
	}
}

void IrSend::sendKase( byte *p )
{
	int     i, j ;
	byte    c ;

	// start marker
	IrHighOut( UStoCNT( KASE_START_H ) ) ;
	IrLowOut ( UStoCNT( KASE_START_L ) ) ;

	// body
	for( i = 0 ; i < 6 ; i++ ) {
		c = p[i] ;
		for( j = 0 ; j < 8 ; j++ ) {
			IrHighOut( UStoCNT( KASE_DATA_H ) ) ;
			if( (c & 1) != 0 ) {
				IrLowOut( UStoCNT( KASE_ONE_L ) ) ;
			}
			else {
				IrLowOut( UStoCNT( KASE_ZERO_L ) ) ;
			}
			c = c / 2 ;
		}
	}

	// stop bit
	IrHighOut( UStoCNT( KASE_DATA_H ) ) ;

	delayMicroseconds( 75000ul ) ;
}

void IrSend::sendNec( byte *p )
{
	int     i, j ;
	byte    c ;

	unsigned long  time = micros() ;

	// start marker
	IrHighOut( UStoCNT( NEC_START_H ) ) ;
	IrLowOut ( UStoCNT( NEC_START_L ) ) ;

	// body
	for( i = 0 ; i < 4 ; i++ ) {
		c = p[i] ;
		for( j = 0 ; j < 8 ; j++ ) {
			IrHighOut( UStoCNT( NEC_DATA_H ) ) ;
			if( (c & 1) != 0 ) {
				IrLowOut( UStoCNT( NEC_ONE_L ) ) ;
			}
			else {
				IrLowOut( UStoCNT( NEC_ZERO_L ) ) ;
			}
			c = c / 2 ;
		}
	}

	// stop bit
	IrHighOut( UStoCNT( NEC_DATA_H ) ) ;

	delayMicroseconds( 108000ul - (micros() - time) ) ;
}

void IrSend::sendNecRepeat( void )
{
	unsigned long  time = micros() ;

	// start marker
	IrHighOut( UStoCNT( NEC_START_H ) ) ;
	// repeat
	IrLowOut ( UStoCNT( NEC_REPEAT_L ) ) ;
	// stop bit
	IrHighOut( UStoCNT( NEC_DATA_H ) ) ;

	delayMicroseconds( 108000ul - (micros() - time) ) ;
}

void IrSend::sendSony( byte *p, byte len )
{
	int     i, j ;
	byte    c, f ;

	for( j = 0 ; j < 3 ; j++ ) {
		unsigned long  time = micros() ;

		// start marker
		IrHighOut( UStoCNT( SONY_START_H ) ) ;

		// body
		f = 0 ;
		for( i = 0 ; i < len ; i++ ) {
			if( f == 0 ) {
				f = 1 ;
				c = p[i/8] ;
			}
			IrLowOut( UStoCNT( SONY_DATA_L ) ) ;
			if( (c & f) != 0 ) {
				IrHighOut( UStoCNT( SONY_ONE_H ) ) ;
			}
			else {
				IrHighOut( UStoCNT( SONY_ZERO_H ) ) ;
			}
			f <<= 1 ;
		}
		delayMicroseconds( 45000ul - (micros() - time) ) ;
	}
}


void IrSend::IrHighOut( word n )
{
  word   i ;
  for( i = 0 ; i < n ; i++ ) {
    digitalWrite( led, HIGH ) ;
    delayMicroseconds( IrOnTime ) ;
    digitalWrite( led, LOW ) ;
    delayMicroseconds( IrOffTime ) ;
  }
}

void IrSend::IrLowOut( word n )
{
  word   i ;
  for( i = 0 ; i < n ; i++ ) {
    digitalWrite( led, LOW ) ;
    delayMicroseconds( IrOnTime ) ;
    digitalWrite( led, LOW ) ;
    delayMicroseconds( IrOffTime ) ;
  }
}
